package com.tendencies.jcr.control;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import java.net.URLEncoder;

import javax.jcr.AccessDeniedException;
import javax.jcr.InvalidItemStateException;
import javax.jcr.ItemExistsException;
import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.Property;
import javax.jcr.ReferentialIntegrityException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.ValueFormatException;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.version.VersionException;

import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.ExoContainerContext;
import org.exoplatform.download.DownloadResource;
import org.exoplatform.download.DownloadService;
import org.exoplatform.download.InputStreamDownloadResource;
import org.richfaces.model.TreeNodeImpl;

import com.tendencies.jcr.connection.RepositoryConnector;
import com.tendencies.jcr.treeview.NodeData;

public class ContentManager {
	
	private Node clientContentNode = null;

	public ContentManager (Node clientContentNode) {
		this.clientContentNode = clientContentNode;
	}
	
	public boolean addClient (String CNPJ) {
		
		if (clientExists(CNPJ)) return false;
		
		String dashlessCNPJ = makeCNPJDashless(CNPJ);
		
		return NodeManager.addNode(clientContentNode, dashlessCNPJ, "nt:folder");
		
	}
	
	public boolean removeClient (String CNPJ) {
		
		if (!clientExists(CNPJ)) return false;
		
		String dashlessCNPJ = makeCNPJDashless(CNPJ);
		
		Node clientNode = NodeManager.getNode(clientContentNode, makeCNPJDashless(CNPJ));
		
		try {
			clientNode.remove();
		} catch (Exception e) {
			try {
				clientContentNode.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			e.printStackTrace();
			return false;
		}
		
		try {
			clientContentNode.save();
		} catch (Exception e) {
			try {
				clientContentNode.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			e.printStackTrace();
		}
		
		return true;
		
	}
	
	public boolean addCategory (String CNPJ, String category) {
		
		if (!clientExists(CNPJ)) return false;
		
		if (categoryExists(CNPJ, category)) return false;
		
		Node clientNode = NodeManager.getNode(clientContentNode, makeCNPJDashless(CNPJ));
		
		NodeManager.addNode(clientNode, category, "nt:folder");
		
		return true;
		
	}
	
	public boolean removeCategory (String CNPJ, String category) {
		
		if (!clientExists(CNPJ)) return false;
		
		if (!categoryExists(CNPJ, category)) return false;
		
		Node categoryNode = NodeManager.getNode(clientContentNode, makeCNPJDashless(CNPJ) + "/" + category);
		
		try {
			categoryNode.remove();
		} catch (Exception e) {
			try {
				clientContentNode.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			e.printStackTrace();
			return false;
		}
		
		try {
			clientContentNode.getSession().save();
		} catch (Exception e) {
			try {
				clientContentNode.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			e.printStackTrace();
			return false;
		}
		
		return true;
		
	}
	
	public boolean clientExists (String CNPJ) {
		
		if (NodeManager.getNode(clientContentNode, makeCNPJDashless(CNPJ)) != null) return true;
			
		else return false;
			
	}
	
	public boolean categoryExists (String CNPJ, String category) {
		
		if (NodeManager.getNode(clientContentNode, makeCNPJDashless(CNPJ) + "/" + category) != null) return true;
		
		else return false;
		
	}
	
	public boolean contentExists (String contentPath) {
		try {
			clientContentNode.getNode(contentPath);
		} catch (PathNotFoundException e) {
			e.printStackTrace();
			return false;
		} catch (RepositoryException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}
	
	public boolean addContent (File file, String mimeType, String CNPJ, String contentCategory) {
		
		String dashlessCNPJ = makeCNPJDashless(CNPJ);
		
		if (contentExists(dashlessCNPJ + "/" + contentCategory + "/" + file.getName())) return false;
		
		if (!clientExists(CNPJ)) addClient(CNPJ);
		
		Node categoryNode = NodeManager.getNode(clientContentNode, dashlessCNPJ + "/" + contentCategory);
		
		if (categoryNode == null) {
			NodeManager.addNode(clientContentNode, dashlessCNPJ + "/" + contentCategory);
			categoryNode = NodeManager.getNode(clientContentNode, dashlessCNPJ + "/" + contentCategory);
		}
		
		return NodeManager.saveFile(categoryNode, file, mimeType);
		
	}
	
	public boolean addContent (byte[] data, String name, String mimeType, String CNPJ, String contentCategory) {
		
		String dashlessCNPJ = makeCNPJDashless(CNPJ);
		
		if (contentExists(dashlessCNPJ + "/" + contentCategory + "/" + name)) return false;
		
		if (!clientExists(CNPJ)) addClient(CNPJ);
		
		Node categoryNode = NodeManager.getNode(clientContentNode, dashlessCNPJ + "/" + contentCategory);
		
		if (categoryNode == null) {
			NodeManager.addNode(clientContentNode, dashlessCNPJ + "/" + contentCategory);
			categoryNode = NodeManager.getNode(clientContentNode, dashlessCNPJ + "/" + contentCategory);
		}
		
		return NodeManager.saveFile(categoryNode, data, name, mimeType);
		
	}
		
	public boolean removeContent (String contentPath) {
		Node contentNode = null;
		try {
			contentNode = clientContentNode.getNode(contentPath);
		} catch (PathNotFoundException e) {
			e.printStackTrace();
			return false;
		} catch (RepositoryException e) {
			e.printStackTrace();
			return false;
		}
		try {
			contentNode.remove();
		} catch (Exception e) {
			try {
				clientContentNode.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			e.printStackTrace();
		}
		
		try {
			clientContentNode.getSession().save();
		} catch (Exception e) {
			try {
				clientContentNode.refresh(false);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			e.printStackTrace();
			return false;
		}
		return true;
	}
	
	public BufferedInputStream getDownloadStream (String filePath) {
		
		return NodeManager.getFile(NodeManager.getNode(clientContentNode, filePath));
		
	}
	
	public String viewClientContentTree (String CNPJ) {
		
		if (!clientExists(CNPJ)) return null;
		
		String dashlessCNPJ = makeCNPJDashless(CNPJ);
		
		Node clientNode = NodeManager.getNode(clientContentNode, dashlessCNPJ);
		
		return NodeManager.printTree(clientNode);
		
	}

	public String viewContentTree () {
		
		return NodeManager.printTree(clientContentNode);
		
	}
	
	public String makeCNPJDashless (String CNPJ) {
		
		int dashIndex;

		while ((dashIndex = CNPJ.indexOf("/")) != -1) {
			CNPJ = CNPJ.substring(0, dashIndex) + " " + CNPJ.substring(dashIndex + 1);
		}
		
		return CNPJ;
		
	}
	
	public TreeNodeImpl<NodeData> getTree () {
		ExoContainer container = ExoContainerContext.getCurrentContainer();
		DownloadService downloadService = DownloadService.class.cast(container.getComponentInstanceOfType(DownloadService.class));
		TreeNodeImpl<NodeData> tree = new TreeNodeImpl<NodeData>();
		NodeData rootData = new NodeData();
		rootData.setName("Clientes");
		tree.setData(rootData);
		List<Node> clientList = NodeManager.getChildNodes(clientContentNode);
		int clientCount = 0;
		for (Iterator<Node> it = clientList.iterator(); it.hasNext(); ) {
			Node client = it.next();
			TreeNodeImpl<NodeData> clientTI = new TreeNodeImpl<NodeData>();
			String clientPath = "";
			NodeData clientND = new NodeData();
			try {
				clientPath = client.getName();
			} catch (RepositoryException e) {
				e.printStackTrace();
			}
			clientND.setName(clientPath);
			clientND.setType("client");
			clientTI.setData(clientND);
			List<Node> categories = NodeManager.getChildNodes(client);
			int categoriesCount = 0;
			for (Iterator<Node> catIt = categories.iterator(); catIt.hasNext(); ) {
				Node category = catIt.next();
				TreeNodeImpl<NodeData> categoryTI = new TreeNodeImpl<NodeData>();
				String categoryPath = "";
				NodeData categoryND = new NodeData();
				try {
					categoryPath = category.getName();
				} catch (RepositoryException e) {
					e.printStackTrace();
				}
				categoryND.setName(categoryPath);
				categoryND.setType("category");
				categoryND.setClient(clientPath);
				categoryND.setPath(clientPath + "/" + categoryPath);
				categoryTI.setData(categoryND);
				List<Node> files = NodeManager.getChildNodes(category);
				int fileCount = 0;
				for (Iterator<Node> filesIt = files.iterator(); filesIt.hasNext(); ) {
					Node file = filesIt.next();
					TreeNodeImpl<NodeData> fileTI = new TreeNodeImpl<NodeData>();
					Property dataProperty = null;
					Property mimeTypeProperty = null;
					try {
						dataProperty = file.getNode("jcr:content").getProperty("jcr:data");
						mimeTypeProperty = file.getNode("jcr:content").getProperty("jcr:mimeType");
					} catch (PathNotFoundException e1) {
						e1.printStackTrace();
					} catch (RepositoryException e1) {
						e1.printStackTrace();
					}
					String filePath = "";
					NodeData fileND = new NodeData();
					try {
						filePath = file.getName();
					} catch (RepositoryException e) {
						e.printStackTrace();
					}
					InputStream stream = null;
					String mimeType = null;
					try {
						stream = dataProperty.getStream();
						mimeType = mimeTypeProperty.getString();
					} catch (ValueFormatException e) {
						e.printStackTrace();
					} catch (RepositoryException e) {
						e.printStackTrace();
					}
					String downloadLink = "";
					if (stream != null) {
						try {
							DownloadResource downloadResource = new InputStreamDownloadResource(stream, mimeType);
							downloadResource.setDownloadName(filePath);
							downloadLink = downloadService.getDownloadLink(downloadService.addDownloadResource(downloadResource));
							downloadLink = downloadLink.replaceAll("&amp;", "&");
							System.out.println("################### " + filePath + "### " + downloadLink);
						} catch (Exception e) {
							e.printStackTrace();
						} finally {
							try {
								stream.close();
							} catch (IOException e2) {
								e2.printStackTrace();
							}
						}
					}
					fileND.setName(filePath);
					fileND.setType("file");
					fileND.setDownloadLink(downloadLink);
					fileND.setCategory(categoryPath);
					fileND.setClient(clientPath);
					fileND.setPath(clientPath + "/" + categoryPath + "/" + filePath);
					fileTI.setData(fileND);
					categoryTI.addChild(fileCount, fileTI);
					fileCount++;
				}
				clientTI.addChild(categoriesCount, categoryTI);
				categoriesCount++;
			}
			tree.addChild(clientCount, clientTI);
			clientCount++;
		}
		return tree;
	}
	
//	public TreeNodeImpl<NodeData> getClientTree(String CNPJ) {
//		
//	}
	
}
